package Source::Shared::FindLocalMirror::ALTLinux;

use strict;
use warnings;

use Source::Shared::FindLocalMirror;
use Source::Shared::CLI;

our @ISA = qw/Source::Shared::CLI/;
our $VERSION='0.004';

my $def_branch='sisyphus';
our $opt_branch=$def_branch;
my $opt_overlay;
my $opt_subpath;
my $opt_find_all;

sub print_longopt {
    print "  FindLocalMirror::ALTLinux options:
	--branch  branch, default is $opt_branch
	--overlay comma separated list of overlay repos, or ''.
	  default is '' (main repository, for example, p8).
	  Example: ,autoimports,fcimport = p8+autoimports+fcimport
	--subpath files/SRPMS
	--all show all mirrors found (default is the first one)
";
}

our @LONGOPT=(
    'b|branch=s' => \$opt_branch,
    'overlay=s' => \$opt_overlay,
    'subpath=s' => \$opt_subpath,
    'root' => sub {$opt_find_all=0},
    'all' => \$opt_find_all,
    );

my @ALT_DIRS=qw! ALTLinux altlinux ALT . !;

sub find_cli {
    my $branch=$opt_branch;
    my @args=(-branch => $branch);
    push @args, -subpath => $opt_subpath if $opt_subpath;
    push @args, -find_all => 1 if $opt_find_all;
    my @overlays=('');
    @overlays=split(',',$opt_overlay) if $opt_overlay;
    return map {&find(@args, -overlay => $_, @_,)} @overlays;
}

sub find {
    my $opt = { @_ };
    my $branch = $opt->{-branch};
    my $overlay = $opt->{-overlay};
    my $subpath = $opt->{-subpath};
    $branch //= $def_branch;
    $subpath //= '/files/SRPMS';
    $subpath = '/'.$subpath if substr($subpath,0,1) ne '/';
    my @branch_prefixes=($branch, $branch.'/branch','repo/'.$branch.'/release/latest');
    @branch_prefixes=('Sisyphus','sisyphus') if $branch eq 'sisyphus' or $branch eq 'Sisyphus';
    my @path_suffixes=map {$_.$subpath} @branch_prefixes;
    @path_suffixes=map {$overlay.'/'.$_} @path_suffixes if $overlay;
    my @search_path_pattern;
    foreach my $suffix (@path_suffixes) {
	foreach my $altdir (@ALT_DIRS) {
	    push @search_path_pattern, $altdir.'/'.$suffix;
	}
    }
    my @found=&Source::Shared::FindLocalMirror::find_local_mirror(\@search_path_pattern);
    if ($opt->{-root} or ! $opt->{-subpath}) {
	my $remove_tail=-length($subpath);
	@found=map {substr($_,0,$remove_tail)} @found;
    }
    return if !@found;
    return $found[0] if !$opt->{-find_all};
    return @found;
}

1;
