package Source::Shared::FindMirror::ALTLinux;

use strict;
use warnings;

use File::Basename;
use Source::Shared::FindLocalMirror;

our $VERSION=0.003;
our $verbose=0;

our $ALT_URL_PREFIX='http://ftp.altlinux.org/pub/distributions/ALTLinux';
my @altlinux_chain_sisyphus=qw!Sisyphus autoimports/Sisyphus p10/branch p9/branch p8/branch t7/branch p7/branch t6/branch p6/branch 5.1/branch!;
my %altlinux_url_known=map {$_=>1} @altlinux_chain_sisyphus, qw!autoimports/p8 autoimports/p7!;
my %altlinux_chain_by_repo=(
    'Sisyphus' => \@altlinux_chain_sisyphus,
    'sisyphus' => \@altlinux_chain_sisyphus,
    'p10' => [ qw!p10/branch autoimports/p10 p9/branch p8/branch t7/branch p7/branch t6/branch p6/branch 5.1/branch! ],
    'p9' => [ qw!p9/branch autoimports/p9 p8/branch t7/branch p7/branch t6/branch p6/branch 5.1/branch! ],
    'p8' => [ qw!p8/branch autoimports/p8 t7/branch p7/branch t6/branch p6/branch 5.1/branch! ],
    't7' => [ qw!t7/branch p7/branch autoimports/p7 t6/branch p6/branch 5.1/branch! ],
    'p7' => [ qw!p7/branch autoimports/p7 t6/branch p6/branch 5.1/branch! ],
    't6' => [ qw!t6/branch p6/branch 5.1/branch! ],
    'p6' => [ qw!p6/branch 5.1/branch! ],
    '5.1' => [ qw!5.1/branch! ],
    'c9' => [ qw!c9/branch! ],
    'c8.1' => [ qw!c8.1/branch! ],
    'c8' => [ qw!c8/branch! ],
    'c7.1' => [ qw!c7.1/branch! ],
    'c7' => [ qw!c7/branch! ],
    'c6' => [ qw!c6/branch! ],
);

my %altlinux_url_skip_prefix=map {$_=>1} qw!
luabuilder
cpanbuilder
fcimport
mgaimport
pldimport
rosaimport
suseimport
misc
!;

sub get_repo {
    my %opt=@_;
    my $_verbose=$opt{-verbose};
    $_verbose=$verbose if $verbose and (not $_verbose or $verbose > $_verbose);
    $_verbose||=0;
    my $branch=$opt{-branch};
    $branch//='sisyphus';
    my @altpath;

    # prepend hasher dir
    if ($opt{-prepend_hasher}) {
	my $hasher_tmp_path=$ENV{TMP};
	# for RPM_BUILD_ROOT
	$hasher_tmp_path //='/tmp';
	my %prefix = ($ENV{HOME} => '~', $hasher_tmp_path => $hasher_tmp_path);
        HASHER:
	foreach my $location (keys (%prefix)) {
	    if ($location) {
		foreach my $subpath (qw!hasher/repo/SRPMS.hasher!) {
		    if (-d $location.'/'.$subpath) {
			push @altpath, $prefix{$location}.'/'.$subpath;
			push @altpath, $prefix{$location}.'/'.$subpath.'.S' if -d $location.'/'.$subpath.'.S';
		    }
		    #print STDERR "hasher found: $prefix{$location}/$subpath\n";
		    last HASHER;
		}
	    }
	}
    }

    # follow by alt
    my @ROOTS=&Source::Shared::FindLocalMirror::find_local_mirror([qw/ ALTLinux altlinux ALT /]);
    print STDERR 'FindMirror::ALTLinux: found roots: ',join(':',@ROOTS),"\n" if $_verbose > 1 and @ROOTS;
    print STDERR 'FindMirror::ALTLinux: opts: -prepend_path=',($opt{-prepend_path} ? join(':',@{$opt{-prepend_path}}):'(none)'),' -append_repository_chain=',($opt{-append_repository_chain}? 'true': 'false'),"\n" if $verbose > 3 or $verbose > 3;

    my @altchain;
    push @altchain, @{$opt{-prepend_path}} if $opt{-prepend_path};
    my $chain_by_repo=$altlinux_chain_by_repo{$branch};
    if (!$chain_by_repo and @ROOTS) {
	foreach my $try_path ($branch.'/branch',$branch) {
	    if (-d $ROOTS[0].'/'.$try_path) {
		$chain_by_repo=[$try_path];
		warn "FindMirror::ALTLinux: branch $branch is not in database. Please, report a bug\n";
		last;
	    }
	}
    }
    die "FindMirror::ALTLinux: branch $branch is not in database. Please, report a bug" if !$chain_by_repo;
    if ($opt{-append_repository_chain}) {
	push @altchain, @$chain_by_repo;
    } else {
	push @altchain, $chain_by_repo->[0];
    }

    if (@ROOTS) {
	foreach my $pathentry (@altchain) {
	    foreach my $ROOT (@ROOTS) {
		my $dir_to_add=$ROOT.'/'.$pathentry.'/files/SRPMS';
		push @altpath, $dir_to_add  if -d $dir_to_add;
	    }
	}
    } else {
	print STDERR "repo (ALT Linux): local mirror not found, using web...\n" if $_verbose;
	foreach my $pathentry (@altchain) {
	    push @altpath, &__to_url($pathentry, $_verbose);
	}
    }
    print STDERR 'FindMirror::ALTLinux: altpath: ',join(':',@altpath),"\n" if $_verbose > 2 or $verbose > 2 and @altpath;
    return @altpath;
}

sub __to_url {
    my ($pathentry,$_verbose)=@_;
    return if $altlinux_url_skip_prefix{$pathentry};
    return if $altlinux_url_skip_prefix{dirname($pathentry)};
    return $ALT_URL_PREFIX.'/'.$pathentry.'/files/SRPMS' if $altlinux_url_known{$pathentry};
    print STDERR "repo (ALT Linux): skipping $pathentry\n" if $_verbose;
    return;
}

1;
